"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const monaco = require("monaco-editor");
const throttle = require("lodash.throttle");
const commands_1 = require("./commands");
const util_1 = require("./util");
const basicInput_1 = require("./basicInput");
const state_1 = require("./state");
const { TextEditorCursorBlinkingStyle, TextEditorCursorStyle, } = monaco.editor;
class EmacsExtension {
    constructor(editor) {
        this._disposables = [];
        this._inSelectionMode = false;
        this._changeDisposable = null;
        this._state = new state_1.State();
        this._onDidMarkChange = new util_1.Emitter();
        this.onDidMarkChange = this._onDidMarkChange.event;
        this._onDidChangeKey = new util_1.Emitter();
        this.onDidChangeKey = this._onDidChangeKey.event;
        this._editor = editor;
        const config = editor.getConfiguration().viewInfo;
        this._intialCursorType = TextEditorCursorStyle[config.cursorStyle].toLowerCase();
        this._intialCursorBlinking = TextEditorCursorBlinkingStyle[config.cursorBlinking].toLowerCase();
        this._basicInputWidget = new basicInput_1.BasicInputWidget();
    }
    start() {
        if (this._disposables.length) {
            return;
        }
        this.addListeners();
        this._editor.updateOptions({
            cursorStyle: TextEditorCursorStyle[TextEditorCursorStyle.Block].toLowerCase(),
            cursorBlinking: TextEditorCursorBlinkingStyle[TextEditorCursorBlinkingStyle.Blink].toLowerCase(),
        });
        this._editor.addOverlayWidget(this._basicInputWidget);
    }
    get state() {
        return this._state;
    }
    getEditor() {
        return this._editor;
    }
    addListeners() {
        this._disposables.push(this._editor.onKeyDown(this.onKeyDown.bind(this)));
        this._throttledScroll = throttle(this.onEditorScroll.bind(this), 500);
        this._disposables.push(this._editor.onDidScrollChange(this._throttledScroll));
    }
    cancelKey(ev) {
        ev.preventDefault();
        ev.stopPropagation();
    }
    onKeyDown(ev) {
        let key = util_1.monacoToEmacsKey(ev);
        if (!key) {
            return;
        }
        key = this._state.updateAndGetKey(key);
        this._onDidChangeKey.fire(this._state.getReadableState());
        if (key === state_1.EAT_UP_KEY) {
            this._onDidChangeKey.fire(this._state.getReadableState());
            this.cancelKey(ev);
            return;
        }
        const command = commands_1.COMMANDS[key];
        if (!command) {
            this._onDidChangeKey.fire(this._state.getReadableState());
            return;
        }
        this.cancelKey(ev);
        commands_1.executeCommand(this, command, this._state.getInputBuffer());
        this._state.updateStateOnExecution();
    }
    onEditorScroll() {
        const { height } = this._editor.getLayoutInfo();
        const layout = this._editor.getScrolledVisiblePosition(this._editor.getPosition());
        if (layout.top >= 0 && layout.top <= height) {
            return;
        }
        const ranges = this._editor.getVisibleRanges();
        if (!ranges.length) {
            return;
        }
        let newPos;
        const sel = this._editor.getSelection();
        if (layout.top < 0) {
            newPos = new monaco.Position(ranges[0].getStartPosition().lineNumber, 1);
        }
        else if (layout.top > height) {
            newPos = new monaco.Position(ranges[ranges.length - 1].getEndPosition().lineNumber, 1);
        }
        if (this._inSelectionMode) {
            this._editor.setSelection(monaco.Selection.fromPositions(sel.getStartPosition(), newPos));
        }
        else {
            this._editor.setPosition(newPos);
        }
    }
    onContentChange() {
        this.selectionMode = false;
    }
    clearState() {
        this._state.updateStateOnExecution();
    }
    set selectionMode(mode) {
        if (mode === this._inSelectionMode) {
            return;
        }
        this._inSelectionMode = mode;
        if (mode) {
            this._changeDisposable = this._editor.onDidChangeModelContent(this.onContentChange.bind(this));
        }
        else if (this._changeDisposable) {
            this._changeDisposable.dispose();
            this._changeDisposable = null;
        }
        this._onDidMarkChange.fire(mode);
    }
    get selectionMode() {
        return this._inSelectionMode;
    }
    getCursorAnchor() {
        const sel = this._editor.getSelection();
        const dir = sel.getDirection();
        return dir === monaco.SelectionDirection.LTR ? sel.getEndPosition() : sel.getStartPosition();
    }
    getBasicInput(message) {
        return this._basicInputWidget.getInput(message);
    }
    dispose() {
        this._disposables.forEach(d => d.dispose());
        this._disposables = undefined;
        if (this._changeDisposable) {
            this._changeDisposable.dispose();
            this._changeDisposable = null;
        }
        this._editor.updateOptions({
            cursorStyle: this._intialCursorType,
            cursorBlinking: this._intialCursorBlinking,
        });
        this._editor.removeOverlayWidget(this._basicInputWidget);
        this._basicInputWidget.dispose();
        this._throttledScroll.cancel();
        this._state = null;
    }
}
exports.EmacsExtension = EmacsExtension;
